@@ -9,8 +9,6 @@ import numpy as np
 from tlz import merge
 
 from dask.array.core import Array
-from dask.array.numpy_compat import NUMPY_GE_122
-from dask.array.numpy_compat import percentile as np_percentile
 from dask.base import tokenize
 from dask.highlevelgraph import HighLevelGraph
 
@@ -23,7 +21,7 @@ def _percentile(a, q, method="linear"):
     if isinstance(q, Iterator):
         q = list(q)
     if a.dtype.name == "category":
-        result = np_percentile(a.cat.codes, q, method=method)
+        result = np.percentile(a.cat.codes, q, method=method)
         import pandas as pd
 
         return pd.Categorical.from_codes(result, a.dtype.categories, a.dtype.ordered), n
@@ -36,22 +34,17 @@ def _percentile(a, q, method="linear"):
     if np.issubdtype(a.dtype, np.datetime64):
         values = a
         if type(a).__name__ in ("Series", "Index"):
-            from dask.dataframe._compat import PANDAS_GE_200
-
-            if PANDAS_GE_200:
-                a2 = values.astype("i8")
-            else:
-                a2 = values.view("i8")
+            a2 = values.astype("i8")
         else:
             a2 = values.view("i8")
-        result = np_percentile(a2, q, method=method).astype(values.dtype)
+        result = np.percentile(a2, q, method=method).astype(values.dtype)
         if q[0] == 0:
             # https://github.com/dask/dask/issues/6864
             result[0] = min(result[0], values.min())
         return result, n
     if not np.issubdtype(a.dtype, np.number):
         method = "nearest"
-    return np_percentile(a, q, method=method), n
+    return np.percentile(a, q, method=method), n
 
 
 def _tdigest_chunk(a):
@@ -83,7 +76,7 @@ def percentile(a, q, method="linear", internal_method="default", **kwargs):
         0 and 100 inclusive.
     method : {'linear', 'lower', 'higher', 'midpoint', 'nearest'}, optional
         The interpolation method to use when the desired percentile lies
-        between two data points ``i < j``. Only valid for ``method='dask'``.
+        between two data points ``i < j``. Only valid for ``internal_method='dask'``.
 
         - 'linear': ``i + (j - i) * fraction``, where ``fraction``
           is the fractional part of the index surrounded by ``i``
@@ -120,18 +113,16 @@ def percentile(a, q, method="linear", internal_method="default", **kwargs):
 
     if method in allowed_internal_methods:
         warnings.warn(
-            "In Dask 2022.1.0, the `method=` argument was renamed to `internal_method=`",
+            "The `method=` argument was renamed to `internal_method=`",
             FutureWarning,
         )
         internal_method = method
 
     if "interpolation" in kwargs:
-        if NUMPY_GE_122:
-            warnings.warn(
-                "In Dask 2022.1.0, the `interpolation=` argument to percentile was renamed to "
-                "`method= ` ",
-                FutureWarning,
-            )
+        warnings.warn(
+            "The `interpolation=` argument to percentile was renamed to " "`method= ` ",
+            FutureWarning,
+        )
         method = kwargs.pop("interpolation")
 
     if kwargs:
