@@ -1,28 +1,27 @@
 from __future__ import annotations
 
-import atexit
+import copyreg
 import dataclasses
 import datetime
+import decimal
 import hashlib
 import inspect
-import os
 import pathlib
 import pickle
-import sys
-import threading
+import types
 import uuid
 import warnings
 from collections import OrderedDict
-from collections.abc import Callable, Iterator, Mapping
+from collections.abc import Hashable, Iterable, Iterator, Mapping
 from concurrent.futures import Executor
-from contextlib import contextmanager
+from contextlib import contextmanager, suppress
 from contextvars import ContextVar
-from enum import Enum
-from functools import partial, wraps
+from functools import partial
 from numbers import Integral, Number
 from operator import getitem
-from typing import TYPE_CHECKING, Any, Literal, Protocol
+from typing import Any, Literal, TypeVar
 
+import cloudpickle
 from tlz import curry, groupby, identity, merge
 from tlz.functoolz import Compose
 
@@ -33,7 +32,7 @@ from dask.core import get as simple_get
 from dask.core import literal, quote
 from dask.hashing import hash_buffer_hex
 from dask.system import CPU_COUNT
-from dask.typing import SchedulerGetCallable
+from dask.typing import Key, SchedulerGetCallable
 from dask.utils import (
     Dispatch,
     apply,
@@ -43,6 +42,26 @@ from dask.utils import (
     shorten_traceback,
 )
 
+_DistributedClient = None
+_get_distributed_client = None
+_DISTRIBUTED_AVAILABLE = None
+
+
+def _distributed_available() -> bool:
+    # Lazy import in get_scheduler can be expensive
+    global _DistributedClient, _get_distributed_client, _DISTRIBUTED_AVAILABLE
+    if _DISTRIBUTED_AVAILABLE is not None:
+        return _DISTRIBUTED_AVAILABLE  # type: ignore[unreachable]
+    try:
+        from distributed import Client as _DistributedClient
+        from distributed.worker import get_client as _get_distributed_client
+
+        _DISTRIBUTED_AVAILABLE = True
+    except ImportError:
+        _DISTRIBUTED_AVAILABLE = False
+    return _DISTRIBUTED_AVAILABLE
+
+
 __all__ = (
     "DaskMethodsMixin",
     "annotate",
@@ -60,46 +79,6 @@ __all__ = (
     "clone_key",
 )
 
-if TYPE_CHECKING:
-    from _typeshed import ReadableBuffer
-
-
-def _clean_traceback_hook(func):
-    @wraps(func)
-    def wrapper(exc_type, exc, tb):
-        tb = shorten_traceback(tb)
-        return func(exc_type, exc.with_traceback(tb), tb)
-
-    return wrapper
-
-
-def _clean_ipython_traceback(self, etype, value, tb, tb_offset=None):
-    short_tb = shorten_traceback(tb)
-    short_exc = value.with_traceback(short_tb)
-    stb = self.InteractiveTB.structured_traceback(
-        etype, short_exc, short_tb, tb_offset=tb_offset
-    )
-    self._showtraceback(etype, short_exc, stb)
-
-
-try:
-    from IPython import get_ipython
-except ImportError:
-    pass
-else:
-    # if we're running in ipython, customize exception handling
-    ip = get_ipython()
-    if ip is not None:
-        ip.set_custom_exc((Exception,), _clean_ipython_traceback)
-
-
-def _restore_excepthook():
-    sys.excepthook = original_excepthook
-
-
-original_excepthook = sys.excepthook
-sys.excepthook = _clean_traceback_hook(sys.excepthook)
-
 _annotations: ContextVar[dict[str, Any]] = ContextVar("annotations", default={})
 
 
@@ -249,16 +228,32 @@ def is_dask_collection(x) -> bool:
     implementation of the protocol.
 
     """
-    try:
-        return x.__dask_graph__() is not None
-    except (AttributeError, TypeError):
+    if (
+        isinstance(x, type)
+        or not hasattr(x, "__dask_graph__")
+        or not callable(x.__dask_graph__)
+    ):
         return False
 
+    pkg_name = getattr(type(x), "__module__", "").split(".")[0]
+    if pkg_name in ("dask_expr", "dask_cudf"):
+        # Temporary hack to avoid graph materialization. Note that this won't work with
+        # dask_expr.array objects wrapped by xarray or pint. By the time dask_expr.array
+        # is published, we hope to be able to rewrite this method completely.
+        # Read: https://github.com/dask/dask/pull/10676
+        return True
+
+    # xarray, pint, and possibly other wrappers always define a __dask_graph__ method,
+    # but it may return None if they wrap around a non-dask object.
+    # In all known dask collections other than dask-expr,
+    # calling __dask_graph__ is cheap.
+    return x.__dask_graph__() is not None
+
 
 class DaskMethodsMixin:
     """A mixin adding standard dask collection methods"""
 
-    __slots__ = ()
+    __slots__ = ("__weakref__",)
 
     def visualize(self, filename="mydask", format=None, optimize_graph=False, **kwargs):
         """Render the computation of this object's task graph using graphviz.
@@ -290,7 +285,7 @@ class DaskMethodsMixin:
 
         Returns
         -------
-        result : IPython.diplay.Image, IPython.display.SVG, or None
+        result : IPython.display.Image, IPython.display.SVG, or None
             See dask.dot.dot_graph for more information.
 
         See Also
@@ -388,13 +383,11 @@ class DaskMethodsMixin:
             raise ImportError(
                 "Using async/await with dask requires the `distributed` package"
             ) from e
-        from tornado import gen
 
-        @gen.coroutine
-        def f():
+        async def f():
             if futures_of(self):
-                yield wait(self)
-            raise gen.Return(self)
+                await wait(self)
+            return self
 
         return f().__await__()
 
@@ -545,7 +538,11 @@ def unpack_collections(*args, traverse=True):
         dsk[collections_token] = quote(results)
         return simple_get(dsk, out)
 
-    return collections, repack
+    # The original `collections` is kept alive by the closure
+    # This causes the collection to be only freed by the garbage collector
+    collections2 = list(collections)
+    collections.clear()
+    return collections2, repack
 
 
 def optimize(*args, traverse=True, **kwargs):
@@ -584,9 +581,9 @@ def optimize(*args, traverse=True, **kwargs):
     >>> a2, b2 = dask.optimize(a, b)
 
     >>> a2.compute() == a.compute()
-    True
+    np.True_
     >>> b2.compute() == b.compute()
-    True
+    np.True_
     """
     collections, repack = unpack_collections(*args, traverse=traverse)
     if not collections:
@@ -639,12 +636,12 @@ def compute(
     >>> a = da.arange(10, chunks=2).sum()
     >>> b = da.arange(10, chunks=2).mean()
     >>> dask.compute(a, b)
-    (45, 4.5)
+    (np.int64(45), np.float64(4.5))
 
     By default, dask objects inside python collections will also be computed:
 
     >>> dask.compute({'a': a, 'b': b, 'c': 1})
-    ({'a': 45, 'b': 4.5, 'c': 1},)
+    ({'a': np.int64(45), 'b': np.float64(4.5), 'c': 1},)
     """
 
     collections, repack = unpack_collections(*args, traverse=traverse)
@@ -663,7 +660,9 @@ def compute(
         keys.append(x.__dask_keys__())
         postcomputes.append(x.__dask_postcompute__())
 
-    results = schedule(dsk, keys, **kwargs)
+    with shorten_traceback():
+        results = schedule(dsk, keys, **kwargs)
+
     return repack([f(r, *a) for r, (f, a) in zip(results, postcomputes)])
 
 
@@ -742,7 +741,7 @@ def visualize(
 
     Returns
     -------
-    result : IPython.diplay.Image, IPython.display.SVG, or None
+    result : IPython.display.Image, IPython.display.SVG, or None
         See dask.dot.dot_graph for more information.
 
     See Also
@@ -759,7 +758,30 @@ def visualize(
 
     dsk = dict(collections_to_dsk(args, optimize_graph=optimize_graph))
 
+    return visualize_dsk(
+        dsk=dsk,
+        filename=filename,
+        traverse=traverse,
+        optimize_graph=optimize_graph,
+        maxval=maxval,
+        engine=engine,
+        **kwargs,
+    )
+
+
+def visualize_dsk(
+    dsk,
+    filename="mydask",
+    traverse=True,
+    optimize_graph=False,
+    maxval=None,
+    o=None,
+    engine: Literal["cytoscape", "ipycytoscape", "graphviz"] | None = None,
+    limit=None,
+    **kwargs,
+):
     color = kwargs.get("color")
+    from dask.order import diagnostics, order
 
     if color in {
         "order",
@@ -773,12 +795,20 @@ def visualize(
         "memoryincreases",
         "memorydecreases",
         "memorypressure",
+        "critical",
+        "cpath",
     }:
         import matplotlib.pyplot as plt
 
-        from dask.order import diagnostics, order
+        if o is None:
+            o_stats = order(dsk, return_stats=True)
+            o = {k: v.priority for k, v in o_stats.items()}
+        elif isinstance(next(iter(o.values())), int):
+            o_stats = order(dsk, return_stats=True)
+        else:
+            o_stats = o
+            o = {k: v.priority for k, v in o.items()}
 
-        o = order(dsk)
         try:
             cmap = kwargs.pop("cmap")
         except KeyError:
@@ -808,11 +838,15 @@ def visualize(
                     key: max(0, val.num_data_when_released - val.num_data_when_run)
                     for key, val in info.items()
                 }
-            else:  # memorydecreases
+            elif color.endswith("memorydecreases"):
                 values = {
                     key: max(0, val.num_data_when_run - val.num_data_when_released)
                     for key, val in info.items()
                 }
+            elif color.split("-")[-1] in {"critical", "cpath"}:
+                values = {key: val.critical_path for key, val in o_stats.items()}
+            else:
+                raise NotImplementedError(color)
 
             if color.startswith("order-"):
 
@@ -856,7 +890,6 @@ def visualize(
                 engine = "cytoscape"
             except ImportError:
                 pass
-
     if engine == "graphviz":
         from dask.dot import dot_graph
 
@@ -964,7 +997,9 @@ def persist(*args, traverse=True, optimize_graph=True, scheduler=None, **kwargs)
         keys.extend(a_keys)
         postpersists.append((rebuild, a_keys, state))
 
-    results = schedule(dsk, keys, **kwargs)
+    with shorten_traceback():
+        results = schedule(dsk, keys, **kwargs)
+
     d = dict(zip(keys, results))
     results2 = [r({k: d[k] for k in ks}, *s) for r, ks, s in postpersists]
     return repack(results2)
@@ -973,33 +1008,101 @@ def persist(*args, traverse=True, optimize_graph=True, scheduler=None, **kwargs)
 ############
 # Tokenize #
 ############
+class TokenizationError(RuntimeError):
+    pass
 
 
-class _HashFactory(Protocol):
-    def __call__(
-        self, string: ReadableBuffer = b"", *, usedforsecurity: bool = True
-    ) -> hashlib._Hash:
-        ...
-
-
-# Pass `usedforsecurity=False` to support FIPS builds of Python
-def _md5(x: ReadableBuffer, _hashlib_md5: _HashFactory = hashlib.md5) -> hashlib._Hash:
-    return _hashlib_md5(x, usedforsecurity=False)
-
-
-def tokenize(*args, **kwargs):
+def tokenize(
+    *args: object, ensure_deterministic: bool | None = None, **kwargs: object
+) -> str:
     """Deterministic token
 
     >>> tokenize([1, 2, '3'])
-    '7d6a880cd9ec03506eee6973ff551339'
+    '06961e8de572e73c2e74b51348177918'
 
     >>> tokenize('Hello') == tokenize('Hello')
     True
+
+    Parameters
+    ----------
+    args, kwargs:
+        objects to tokenize
+    ensure_deterministic: bool, optional
+        If True, raise TokenizationError if the objects cannot be deterministically
+        tokenized, e.g. two identical objects will return different tokens.
+        Defaults to the `tokenize.ensure-deterministic` configuration parameter.
     """
-    hasher = _md5(str(tuple(map(normalize_token, args))).encode())
-    if kwargs:
-        hasher.update(str(normalize_token(kwargs)).encode())
-    return hasher.hexdigest()
+    with _seen_ctx(reset=True), _ensure_deterministic_ctx(ensure_deterministic):
+        token: object = _normalize_seq_func(args)
+        if kwargs:
+            token = token, _normalize_seq_func(sorted(kwargs.items()))
+
+    # Pass `usedforsecurity=False` to support FIPS builds of Python
+    return hashlib.md5(str(token).encode(), usedforsecurity=False).hexdigest()
+
+
+# tokenize.ensure-deterministic flag, potentially overridden by tokenize()
+_ensure_deterministic: ContextVar[bool] = ContextVar("_ensure_deterministic")
+
+# Circular reference breaker used by _normalize_seq_func.
+# This variable is recreated anew every time you call tokenize(). Note that this means
+# that you could call tokenize() from inside tokenize() and they would be fully
+# independent.
+#
+# It is a map of {id(obj): (<first seen incremental int>, obj)} which causes an object
+# to be tokenized as ("__seen", <incremental>) the second time it's encountered while
+# traversing collections. A strong reference to the object is stored in the context to
+# prevent ids from being reused by different objects.
+_seen: ContextVar[dict[int, tuple[int, object]]] = ContextVar("_seen")
+
+
+@contextmanager
+def _ensure_deterministic_ctx(ensure_deterministic: bool | None) -> Iterator[bool]:
+    try:
+        ensure_deterministic = _ensure_deterministic.get()
+        # There's a call of tokenize() higher up in the stack
+        tok = None
+    except LookupError:
+        # Outermost tokenize(), or normalize_token() was called directly
+        if ensure_deterministic is None:
+            ensure_deterministic = config.get("tokenize.ensure-deterministic")
+        tok = _ensure_deterministic.set(ensure_deterministic)
+
+    try:
+        yield ensure_deterministic
+    finally:
+        if tok:
+            tok.var.reset(tok)
+
+
+def _maybe_raise_nondeterministic(msg: str) -> None:
+    with _ensure_deterministic_ctx(None) as ensure_deterministic:
+        if ensure_deterministic:
+            raise TokenizationError(msg)
+
+
+@contextmanager
+def _seen_ctx(reset: bool) -> Iterator[dict[int, tuple[int, object]]]:
+    if reset:
+        # It is important to reset the token on tokenize() to avoid artifacts when
+        # it is called recursively
+        seen: dict[int, tuple[int, object]] = {}
+        tok = _seen.set(seen)
+    else:
+        try:
+            seen = _seen.get()
+            tok = None
+        except LookupError:
+            # This is for debug only, for when normalize_token is called outside of
+            # tokenize()
+            seen = {}
+            tok = _seen.set(seen)
+
+    try:
+        yield seen
+    finally:
+        if tok:
+            tok.var.reset(tok)
 
 
 normalize_token = Dispatch()
@@ -1010,10 +1113,10 @@ normalize_token.register(
         str,
         bytes,
         type(None),
-        type,
         slice,
         complex,
         type(Ellipsis),
+        decimal.Decimal,
         datetime.date,
         datetime.time,
         datetime.datetime,
@@ -1024,35 +1127,41 @@ normalize_token.register(
 )
 
 
-@normalize_token.register(dict)
+@normalize_token.register((types.MappingProxyType, dict))
 def normalize_dict(d):
-    return normalize_token(sorted(d.items(), key=str))
+    return "dict", _normalize_seq_func(sorted(d.items(), key=lambda kv: str(kv[0])))
 
 
 @normalize_token.register(OrderedDict)
 def normalize_ordered_dict(d):
-    return type(d).__name__, normalize_token(list(d.items()))
+    return _normalize_seq_func((type(d), list(d.items())))
 
 
 @normalize_token.register(set)
 def normalize_set(s):
-    return normalize_token(sorted(s, key=str))
+    # Note: in some Python version / OS combinations, set order changes every
+    # time you recreate the set (even within the same interpreter).
+    # In most other cases, set ordering is consistent within the same interpreter.
+    return "set", _normalize_seq_func(sorted(s, key=str))
 
 
-def _normalize_seq_func(seq):
-    # Defined outside normalize_seq to avoid unnecessary redefinitions and
-    # therefore improving computation times.
-    try:
-        return list(map(normalize_token, seq))
-    except RecursionError:
-        if not config.get("tokenize.ensure-deterministic"):
-            return uuid.uuid4().hex
-
-        raise RuntimeError(
-            f"Sequence {str(seq)} cannot be deterministically hashed. Please, see "
-            "https://docs.dask.org/en/latest/custom-collections.html#implementing-deterministic-hashing "
-            "for more information"
-        )
+def _normalize_seq_func(seq: Iterable[object]) -> list[object]:
+    with _seen_ctx(reset=False) as seen:
+        out = []
+        for item in seq:
+            if isinstance(item, (str, bytes, int, float, bool, type(None))):
+                # Basic data type. This is just for performance and compactness of the
+                # output. It doesn't need to be a comprehensive list.
+                pass
+            elif id(item) in seen:
+                # May or may not be a circular recursion. Maybe just a double reference.
+                seen_when, _ = seen[id(item)]
+                item = "__seen", seen_when
+            else:
+                seen[id(item)] = len(seen), item
+                item = normalize_token(item)
+            out.append(item)
+        return out
 
 
 @normalize_token.register((tuple, list))
@@ -1065,102 +1174,106 @@ def normalize_literal(lit):
     return "literal", normalize_token(lit())
 
 
-@normalize_token.register(range)
-def normalize_range(r):
-    return list(map(normalize_token, [r.start, r.stop, r.step]))
+@normalize_token.register(Compose)
+def normalize_compose(func):
+    return _normalize_seq_func((func.first,) + func.funcs)
 
 
-@normalize_token.register(Enum)
-def normalize_enum(e):
-    return type(e).__name__, e.name, e.value
+@normalize_token.register((partial, curry))
+def normalize_partial(func):
+    return _normalize_seq_func((func.func, func.args, func.keywords))
+
+
+@normalize_token.register((types.MethodType, types.MethodWrapperType))
+def normalize_bound_method(meth):
+    return normalize_token(meth.__self__), meth.__name__
+
+
+@normalize_token.register(types.BuiltinFunctionType)
+def normalize_builtin_function_or_method(func):
+    # Note: BuiltinMethodType is BuiltinFunctionType
+    self = getattr(func, "__self__", None)
+    if self is not None and not inspect.ismodule(self):
+        return normalize_bound_method(func)
+    else:
+        return normalize_object(func)
 
 
 @normalize_token.register(object)
 def normalize_object(o):
     method = getattr(o, "__dask_tokenize__", None)
-    if method is not None:
+    if method is not None and not isinstance(o, type):
         return method()
 
-    if callable(o):
-        return normalize_function(o)
+    if type(o) is object:
+        return _normalize_pure_object(o)
 
-    if dataclasses.is_dataclass(o):
-        return normalize_dataclass(o)
+    if isinstance(o, type):
+        copyreg._slotnames(o)
 
-    if not config.get("tokenize.ensure-deterministic"):
+    if dataclasses.is_dataclass(o) and not isinstance(o, type):
+        return _normalize_dataclass(o)
+
+    try:
+        return _normalize_pickle(o)
+    except Exception:
+        _maybe_raise_nondeterministic(
+            f"Object {o!r} cannot be deterministically hashed. See "
+            "https://docs.dask.org/en/latest/custom-collections.html#implementing-deterministic-hashing "
+            "for more information."
+        )
         return uuid.uuid4().hex
 
-    raise RuntimeError(
-        f"Object {str(o)} cannot be deterministically hashed. Please, see "
+
+_seen_objects = set()
+
+
+def _normalize_pure_object(o: object) -> tuple[str, int]:
+    _maybe_raise_nondeterministic(
+        "object() cannot be deterministically hashed. See "
         "https://docs.dask.org/en/latest/custom-collections.html#implementing-deterministic-hashing "
-        "for more information"
+        "for more information."
     )
+    # Idempotent, but not deterministic. Make sure that the id is not reused.
+    _seen_objects.add(o)
+    return "object", id(o)
 
 
-function_cache: dict[Callable, Callable | tuple | str | bytes] = {}
-function_cache_lock = threading.Lock()
+def _normalize_pickle(o: object) -> tuple:
+    buffers: list[pickle.PickleBuffer] = []
+    pik: int | None = None
+    pik2: int
 
-
-def normalize_function(func: Callable) -> Callable | tuple | str | bytes:
-    try:
-        return function_cache[func]
-    except KeyError:
-        result = _normalize_function(func)
-        if len(function_cache) >= 500:  # clear half of cache if full
-            with function_cache_lock:
-                if len(function_cache) >= 500:
-                    for k in list(function_cache)[::2]:
-                        del function_cache[k]
-        function_cache[func] = result
-        return result
-    except TypeError:  # not hashable
-        return _normalize_function(func)
-
-
-def _normalize_function(func: Callable) -> tuple | str | bytes:
-    if isinstance(func, Compose):
-        first = getattr(func, "first", None)
-        funcs = reversed((first,) + func.funcs) if first else func.funcs
-        return tuple(normalize_function(f) for f in funcs)
-    elif isinstance(func, (partial, curry)):
-        args = tuple(normalize_token(i) for i in func.args)
-        if func.keywords:
-            kws = tuple(
-                (k, normalize_token(v)) for k, v in sorted(func.keywords.items())
-            )
-        else:
-            kws = None
-        return (normalize_function(func.func), args, kws)
-    else:
-        try:
-            result = pickle.dumps(func, protocol=4)
-            if b"__main__" not in result:  # abort on dynamic functions
-                return result
-        except Exception:
-            pass
-        if not config.get("tokenize.ensure-deterministic"):
+    for mod in [pickle, cloudpickle]:
+        for _ in range(3):
+            buffers.clear()
             try:
-                import cloudpickle
-
-                return cloudpickle.dumps(func, protocol=4)
+                out = mod.dumps(o, protocol=5, buffer_callback=buffers.append)
+                mod.loads(out, buffers=buffers)
+                pik2 = hash_buffer_hex(out)
             except Exception:
-                return str(func)
+                break
+            if pik == pik2:
+                break
+            pik = pik2
         else:
-            raise RuntimeError(
-                f"Function {str(func)} may not be deterministically hashed by "
-                "cloudpickle. See: https://github.com/cloudpipe/cloudpickle/issues/385 "
-                "for more information."
-            )
+            _maybe_raise_nondeterministic("Failed to tokenize deterministically")
+            break
+    if pik is None:
+        _maybe_raise_nondeterministic("Failed to tokenize deterministically")
+        pik = int(uuid.uuid4())
+    return pik, [hash_buffer_hex(buf) for buf in buffers]
 
 
-def normalize_dataclass(obj):
+def _normalize_dataclass(obj):
     fields = [
-        (field.name, getattr(obj, field.name)) for field in dataclasses.fields(obj)
+        (field.name, normalize_token(getattr(obj, field.name, None)))
+        for field in dataclasses.fields(obj)
     ]
-    return (
-        normalize_function(type(obj)),
-        _normalize_seq_func(fields),
-    )
+    params = obj.__dataclass_params__
+    params = [(attr, getattr(params, attr)) for attr in params.__slots__]
+
+    return normalize_object(type(obj)), params, fields
 
 
 @normalize_token.register_lazy("pandas")
@@ -1214,6 +1327,13 @@ def register_pandas():
         data = list(mgr.arrays) + [df.columns, df.index]
         return list(map(normalize_token, data))
 
+    @normalize_token.register(pd.arrays.ArrowExtensionArray)
+    def normalize_extension_array(arr):
+        try:
+            return (type(arr), normalize_token(arr._pa_array))
+        except AttributeError:
+            return (type(arr), normalize_token(arr._data))
+
     @normalize_token.register(pd.api.extensions.ExtensionArray)
     def normalize_extension_array(arr):
         import numpy as np
@@ -1229,6 +1349,51 @@ def register_pandas():
     def normalize_period_dtype(dtype):
         return normalize_token(dtype.name)
 
+    @normalize_token.register(type(pd.NA))
+    def normalize_na(na):
+        return pd.NA
+
+    @normalize_token.register(pd.offsets.BaseOffset)
+    def normalize_offset(offset):
+        return offset.freqstr
+
+
+@normalize_token.register_lazy("pyarrow")
+def register_pyarrow():
+    import pyarrow as pa
+
+    @normalize_token.register(pa.DataType)
+    def normalize_datatype(dt):
+        return pickle.dumps(dt, protocol=4)
+
+    @normalize_token.register(pa.Table)
+    def normalize_table(dt):
+        return (
+            "pa.Table",
+            normalize_token(dt.schema),
+            normalize_token(dt.columns),
+        )
+
+    @normalize_token.register(pa.ChunkedArray)
+    def normalize_chunked_array(arr):
+        return (
+            "pa.ChunkedArray",
+            normalize_token(arr.type),
+            normalize_token(arr.chunks),
+        )
+
+    @normalize_token.register(pa.Array)
+    def normalize_chunked_array(arr):
+        return (
+            "pa.Array",
+            normalize_token(arr.type),
+            normalize_token(arr.buffers()),
+        )
+
+    @normalize_token.register(pa.Buffer)
+    def normalize_chunked_array(buf):
+        return ("pa.Buffer", hash_buffer_hex(buf))
+
 
 @normalize_token.register_lazy("numpy")
 def register_numpy():
@@ -1238,25 +1403,6 @@ def register_numpy():
     def normalize_array(x):
         if not x.shape:
             return (x.item(), x.dtype)
-        if hasattr(x, "mode") and getattr(x, "filename", None):
-            if hasattr(x.base, "ctypes"):
-                offset = (
-                    x.ctypes._as_parameter_.value - x.base.ctypes._as_parameter_.value
-                )
-            else:
-                offset = 0  # root memmap's have mmap object as base
-            if hasattr(
-                x, "offset"
-            ):  # offset numpy used while opening, and not the offset to the beginning of file
-                offset += x.offset
-            return (
-                x.filename,
-                os.path.getmtime(x.filename),
-                x.dtype,
-                x.shape,
-                x.strides,
-                offset,
-            )
         if x.dtype.hasobject:
             try:
                 try:
@@ -1270,70 +1416,29 @@ def register_numpy():
                     # bytes fast-path
                     data = hash_buffer_hex(b"-".join(x.flat))
             except (TypeError, UnicodeDecodeError):
-                try:
-                    data = hash_buffer_hex(pickle.dumps(x, pickle.HIGHEST_PROTOCOL))
-                except Exception:
-                    # pickling not supported, use UUID4-based fallback
-                    if not config.get("tokenize.ensure-deterministic"):
-                        data = uuid.uuid4().hex
-                    else:
-                        raise RuntimeError(
-                            f"``np.ndarray`` with object ``dtype`` {str(x)} cannot "
-                            "be deterministically hashed. Please, see "
-                            "https://docs.dask.org/en/latest/custom-collections.html#implementing-deterministic-hashing "  # noqa: E501
-                            "for more information"
-                        )
+                return normalize_object(x)
         else:
             try:
                 data = hash_buffer_hex(x.ravel(order="K").view("i1"))
             except (BufferError, AttributeError, ValueError):
                 data = hash_buffer_hex(x.copy().ravel(order="K").view("i1"))
-        return (data, x.dtype, x.shape, x.strides)
+        return (data, x.dtype, x.shape)
 
-    @normalize_token.register(np.matrix)
-    def normalize_matrix(x):
-        return type(x).__name__, normalize_array(x.view(type=np.ndarray))
-
-    normalize_token.register(np.dtype, repr)
-    normalize_token.register(np.generic, repr)
+    @normalize_token.register(np.memmap)
+    def normalize_mmap(mm):
+        return hash_buffer_hex(np.ascontiguousarray(mm))
 
     @normalize_token.register(np.ufunc)
-    def normalize_ufunc(x):
+    def normalize_ufunc(func):
         try:
-            name = x.__name__
-            if getattr(np, name) is x:
-                return "np." + name
-        except AttributeError:
-            return normalize_function(x)
-
-    @normalize_token.register(np.random.BitGenerator)
-    def normalize_bit_generator(bg):
-        return normalize_token(bg.state)
-
-
-@normalize_token.register_lazy("scipy")
-def register_scipy():
-    import scipy.sparse as sp
-
-    def normalize_sparse_matrix(x, attrs):
-        return (
-            type(x).__name__,
-            normalize_seq(normalize_token(getattr(x, key)) for key in attrs),
-        )
-
-    for cls, attrs in [
-        (sp.dia_matrix, ("data", "offsets", "shape")),
-        (sp.bsr_matrix, ("data", "indices", "indptr", "blocksize", "shape")),
-        (sp.coo_matrix, ("data", "row", "col", "shape")),
-        (sp.csr_matrix, ("data", "indices", "indptr", "shape")),
-        (sp.csc_matrix, ("data", "indices", "indptr", "shape")),
-        (sp.lil_matrix, ("data", "rows", "shape")),
-    ]:
-        normalize_token.register(cls, partial(normalize_sparse_matrix, attrs=attrs))
-
-    @normalize_token.register(sp.dok_matrix)
-    def normalize_dok_matrix(x):
-        return type(x).__name__, normalize_token(sorted(x.items()))
+            return _normalize_pickle(func)
+        except Exception:
+            _maybe_raise_nondeterministic(
+                f"Cannot tokenize numpy ufunc {func!r}. Please use functions "
+                "of the dask.array.ufunc module instead. See also "
+                "https://docs.dask.org/en/latest/array-numpy-compatibility.html"
+            )
+            return uuid.uuid4().hex
 
 
 def _colorize(t):
@@ -1425,13 +1530,12 @@ def get_scheduler(get=None, scheduler=None, collections=None, cls=None):
         elif isinstance(scheduler, str):
             scheduler = scheduler.lower()
 
-            try:
-                from distributed import Client
-
-                Client.current(allow_global=True)
-                client_available = True
-            except (ImportError, ValueError):
-                client_available = False
+            client_available = False
+            if _distributed_available():
+                assert _DistributedClient is not None
+                with suppress(ValueError):
+                    _DistributedClient.current(allow_global=True)
+                    client_available = True
             if scheduler in named_schedulers:
                 if client_available:
                     warnings.warn(
@@ -1444,9 +1548,8 @@ def get_scheduler(get=None, scheduler=None, collections=None, cls=None):
                     raise RuntimeError(
                         f"Requested {scheduler} scheduler but no Client active."
                     )
-                from distributed.worker import get_client
-
-                return get_client().get
+                assert _get_distributed_client is not None
+                return _get_distributed_client().get
             else:
                 raise ValueError(
                     "Expected one of [distributed, %s]"
@@ -1533,7 +1636,7 @@ def get_collection_names(collection) -> set[str]:
     return {get_name_from_key(k) for k in flatten(collection.__dask_keys__())}
 
 
-def get_name_from_key(key) -> str:
+def get_name_from_key(key: Key) -> str:
     """Given a dask collection's key, extract the collection name.
 
     Parameters
@@ -1553,10 +1656,13 @@ def get_name_from_key(key) -> str:
         return key[0]
     if isinstance(key, str):
         return key
-    raise TypeError(f"Expected str or tuple[str, Hashable, ...]; got {key}")
+    raise TypeError(f"Expected str or a tuple starting with str; got {key!r}")
 
 
-def replace_name_in_key(key, rename: Mapping[str, str]):
+KeyOrStrT = TypeVar("KeyOrStrT", Key, str)
+
+
+def replace_name_in_key(key: KeyOrStrT, rename: Mapping[str, str]) -> KeyOrStrT:
     """Given a dask collection's key, replace the collection name with a new one.
 
     Parameters
@@ -1581,28 +1687,25 @@ def replace_name_in_key(key, rename: Mapping[str, str]):
         return (rename.get(key[0], key[0]),) + key[1:]
     if isinstance(key, str):
         return rename.get(key, key)
-    raise TypeError(f"Expected str or tuple[str, Hashable, ...]; got {key}")
+    raise TypeError(f"Expected str or a tuple starting with str; got {key!r}")
 
 
-def clone_key(key, seed):
+def clone_key(key: KeyOrStrT, seed: Hashable) -> KeyOrStrT:
     """Clone a key from a Dask collection, producing a new key with the same prefix and
     indices and a token which is a deterministic function of the previous key and seed.
 
     Examples
     --------
     >>> clone_key("x", 123)
-    'x-dc2b8d1c184c72c19faa81c797f8c6b0'
+    'x-c4fb64ccca807af85082413d7ef01721'
     >>> clone_key("inc-cbb1eca3bafafbb3e8b2419c4eebb387", 123)
-    'inc-f81b5a88038a2132882aa29a9fcfec06'
+    'inc-bc629c23014a4472e18b575fdaf29ee7'
     >>> clone_key(("sum-cbb1eca3bafafbb3e8b2419c4eebb387", 4, 3), 123)
-    ('sum-fd6be9e9fe07fc232ad576fa997255e8', 4, 3)
+    ('sum-c053f3774e09bd0f7de6044dbc40e71d', 4, 3)
     """
     if isinstance(key, tuple) and key and isinstance(key[0], str):
         return (clone_key(key[0], seed),) + key[1:]
     if isinstance(key, str):
         prefix = key_split(key)
         return prefix + "-" + tokenize(key, seed)
-    raise TypeError(f"Expected str or tuple[str, Hashable, ...]; got {key}")
-
-
-atexit.register(_restore_excepthook)
+    raise TypeError(f"Expected str or a tuple starting with str; got {key!r}")
